// ============================================================================
// Project:	Deductive Database
// Filename:	rel_n_n.h
// Purpose:	Relation with two non-negative integer columns, index over 1st.
// Last Change:	16.11.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2010-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	REL_N_N_INCLUDED
#define REL_N_N_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef MPOOL_INCLUDED
#include "mpool.h"
#endif

#ifndef FLEXARR_INCLUDED
#include "flexarr.h"
#endif

#ifndef REL_INCLUDED
#include "rel.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// REL_N_N_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long REL_N_N_MAGIC = 0x524E4E0AL;	// 'RNN\n'

//=============================================================================
// Class for Relation with two non-negative integer columns:
//=============================================================================

// Non-negative integers are used as ID for strings and for atoms.
// This version has an index on the first column.
// It is assumed that the first column is dense, so we can basically use
// an array as index. However, it is also assumed that a fixed-size array
// is not sufficient, so the relation can be quite big or of an unknown size.

class rel_n_n_c : public rel_c {
	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor:
	rel_n_n_c(str_t rel_name);

	// Destructor:
	~rel_n_n_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// insert: Insert tuple into relation.
	bool insert(int c1, int c2);

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	public:
	// Integrity check:
	str_t	check() const;

	private:
	// Magic number (identifies objects of this class for debugging).
	long	Magic;		// Must be "REL_N_N_MAGIC".
#endif

#ifdef	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	rel_n_n_c(const rel_n_n_c& obj);		// Not implemented
	rel_n_n_c& operator=(const rel_n_n_c& obj);	// Not implemented

//-----------------------------------------------------------------------------
// This class works together with the corresponding cursor class:
//-----------------------------------------------------------------------------

	friend class cur_n_n_c;

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

	// Array indexed by first column, contains second column or list index.
	flexarr_c<int> IndexArr;

	// List nodes to store value sets with >1 element for the 2nd column:
	flexarr_c<int> ValueArr;

//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef rel_n_n_c *rel_n_n_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define REL_N_N_NULL (static_cast<rel_n_n_t>(0))

//=============================================================================
// End of Include File:
//=============================================================================

#endif

